@@ -1,7 +1,6 @@
 import numpy as np
 
-import ashfunc
-from . import bandwidth_selection
+import fastash.ashfunc
 from .whitening_transformer import WhiteningTransformer
 from .hyperplane import Hyperplane
 
@@ -48,7 +47,7 @@ class ASH():
         # BANDWIDTH SELECTION
         self._compute_bandwidth(X)
         
-        self._ret = ashfunc.histograms(X,
+        self._X_digit_uniques, self._P = fastash.ashfunc.compute_bins_P(X,
                                         self._h,
                                         self.q,
                                         self.n_mc,
@@ -67,7 +66,7 @@ class ASH():
         # get indices outside bounds
         # it will be use to cut off the result later
             
-        f = ashfunc.merge_predict(X_digit_uniques = self._X_digit_uniques,
+        f = fastash.ashfunc.merge_predict(X_digit_uniques = self._X_digit_uniques,
                                   P = self._P,
                                   Y = X,
                                   x_min = self._x_min,
@@ -133,7 +132,7 @@ class ASH():
                 # the scott rule is based on gaussian kernel
                 # the support of the gaussian kernel to have 99%
                 # of the density is 2.576
-                self._h = 2.576 * bandwidth_selection.scotts_rule(X)
+                self._h = 2.576 * scotts_rule(X)
             else:
                 raise (ValueError("Unexpected bandwidth selection method."))
         else:
@@ -159,3 +158,22 @@ class ASH():
         """
         for param, value in params.items():
             setattr(self, param, value)
+
+def scotts_rule(X):
+    """
+    Scott's rule according to "Multivariate density estimation", Scott 2015, p.164.
+    The Silverman's rule is exactly the same in "Density estimation for statistics and data analysis", Silverman 1986, p.87.
+    Parameters
+    ----------
+    X : numpy array of shape (n_samples, n_features).
+
+    Returns
+    -------
+    h : float
+        Scotts rule bandwidth. The returned bandwidth should be then factored
+        by the data variance.
+
+    """
+    n = X.shape[0]
+    d = X.shape[1]
+    return(n**(-1/(d + 4)))
\ No newline at end of file
